package de.bsvrz.vew.bmvew.bmvew;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Vector;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ClientApplication;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.dambach.vewdynobj.VerwaltungDynObj;
import de.bsvrz.sys.funclib.operatingMessage.MessageCauser;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageState;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;
import de.bsvrz.vew.bmvew.bmvew.sender.Sender;

public class SystemTest implements ClientReceiverInterface, MutableSetChangeListener
{

  /**
   * Datenverteilerverbindung.
   */
  private static ClientDavInterface _connection = null;

  /**
   * Pid Betriebsmeldungsverwaltung
   */
  private static String _strBetriebsMeldungsVerwaltung = "kv.aoe.nw.nba.koeln.koblenz";

  /**
   * Pid Konfigbereich
   */
  private static String _strKonfigBereich = "kb.default.aoe.nw.nba.koeln.koblenz";

  /**
   * Pid versandobjekt
   */
  private static String _strVersandObjektKex = "versandEmailFaxNw";

  private static Process _prozess = null;

  private static Vector<ResultData> _resultList = null;

  private static Vector<String> _removedList = null;

  private static SystemObject aoe;

  private static MessageCauser causer;

  private static MessageSender instance;

  private static ConfigurationObject _configObj;

  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
    ClientDavParameters parameters = new ClientDavParameters();
    parameters.setDavCommunicationAddress("localhost");
    // parameters.setDavCommunicationAddress("192.0.1.163");
    parameters.setDavCommunicationSubAddress(8083);
    parameters.setUserName("Tester");
    parameters.setUserPassword("geheim");

    _connection = new ClientDavConnection(parameters);
    _connection.connect();
    _connection.login();

    aoe = _connection.getLocalConfigurationAuthority();
    SystemObject user = _connection.getDataModel().getObject("benutzer.tester");
    causer = new MessageCauser(user, "Test", "Tester");
    instance = MessageSender.getInstance();
    ClientApplication app = _connection.getLocalApplicationObject();
    instance.init(_connection, app.getName(), app.getName());

    starteAppliaktionsProzess();

    Thread.sleep(5000);
  }

  private static void starteAppliaktionsProzess() throws IOException
  {
    // String separator = System.getProperty("file.separator");
    String separator = "/";
    String javaHome = System.getProperty("java.home");
    String classPath = System.getProperty("java.class.path");

    ClientDavParameters params = _connection.getClientDavParameters();
    String dav = params.getDavCommunicationAddress() + ":" + params.getDavCommunicationSubAddress();
    String user = params.getUserName();

    String cmdLine = javaHome + separator + "bin" + separator + "java";
    cmdLine += " -cp " + "\"" + classPath + "\"" + " -Xmx128m ";
    cmdLine += "de.bsvrz.vew.bmvew.bmvew.BetriebsMeldungApp";
    cmdLine += " -datenverteiler=" + dav;
    cmdLine += " -benutzer=" + user;
    cmdLine += " -authentifizierung=passwd";
    //cmdLine += " -authentifizierung=C:/kernsoftware-3.4.10-beispielsystem/skripte-dosshell/passwd";
    cmdLine += " -debugLevelStdErrText=CONFIG";
    cmdLine += " -konfigurationsbereich=" + _strKonfigBereich;
    cmdLine += " -konfigurationsobjekt=" + _strBetriebsMeldungsVerwaltung;
    cmdLine += " -versandobjektkex=" + _strVersandObjektKex;
//    cmdLine += " -loeschoffset=" + "\"" + "1m" + "\"";
//    cmdLine += " -groesseinfoanteil=" + "\"" + "10" + "\"";
    cmdLine += " -loeschoffset=10s";
    cmdLine += " -groesseinfoanteil=10";

    _prozess = Runtime.getRuntime().exec(cmdLine);

    new StdErrCatcher().start();

  }

  private static class StdErrCatcher extends Thread
  {

    byte[] buffer = new byte[1024];

    @Override
    public void run()
    {
      while (true)
      {

        try
        {
          if (_prozess.getErrorStream().available() != 0)
          {
            int anzahl = _prozess.getErrorStream().read(buffer);
            System.out.print("" + new String(buffer, 0, anzahl));

          }
          else
          {
            sleep(100);
          }
        }
        catch (IOException e)
        {
          // TODO Auto-generated catch block
          e.printStackTrace(System.out);
        }
        catch (InterruptedException e)
        {
          // TODO Auto-generated catch block
          e.printStackTrace(System.out);
        }

      }
    }
  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
    loescheDynamischesObjekt("meldung.ArchivsystemMeldung3");
    
    if (_prozess != null)    
      _prozess.destroy();

  }

  @Before
  public void setUp() throws Exception
  {
    _configObj = (ConfigurationObject)aoe;

    MutableSet ms = _configObj.getMutableSet("Meldungen");

    if (ms != null)
    {

      ms.addChangeListener(this);
    }
  }

  @After
  public void tearDown() throws Exception
  {
    _configObj = (ConfigurationObject)aoe;

    MutableSet ms = _configObj.getMutableSet("Meldungen");

    if (ms != null)
    {

      ms.removeChangeListener(this);
    }

  }

  @Test(timeout = 10000)
  public void systemTest1() throws Exception
  {
    anmeldenFaxVersandAuftrag();

    parametriereTest1();


    _resultList = new Vector<ResultData>();
    versendeMeldung(1, 0);

    // Warten bis Betriebsmeldungsverwaltung die Antwort publiziert hat
    synchronized (_resultList)
    {
      while (_resultList.size() < 1)
      {
        _resultList.wait();
      }

      ResultData rd = _resultList.get(0);

      // System.out.println(rd);

      Data d = rd.getData();
      String s0 = d.getItem("MeldungsID").getItem("LaufendeNummer").asTextValue().getText();
      String s1 = d.getItem("Empfaenger").asArray().getItem(0).asTextValue().getText();
      String s2 = d.getItem("Text").asTextValue().getText();

      String s = s0 + " " + s1 + " " + s2;

      // berprfen, ob bmvew den Fax-Versand-Auftrag publiziert hat
      assertEquals("Test 1", "0 +497222402209 Dies ist Meldung -> 0", s);
    }
  }

  @Test(timeout = 10000)
  public void systemTest2() throws Exception
  {

    parametriereTest2();

    anmeldenEmailVersandAuftrag();

    _resultList = new Vector<ResultData>();
    versendeMeldung(2, 1);

    // Warten bis Betriebsmeldungsverwaltung die Antwort publiziert hat
    synchronized (_resultList)
    {
      while (_resultList.size() < 1)
      {
        _resultList.wait();
      }

      ResultData rd = _resultList.get(0);

      // System.out.println(rd);

      Data d = rd.getData();
      String s0 = d.getItem("MeldungsID").getItem("LaufendeNummer").asTextValue().getText();
      String s1 = d.getItem("Empfaenger").asTextValue().getText();
      String s2 = d.getItem("Text").asTextValue().getText();

      String s = s0 + " " + s1 + " " + s2;

      // berprfen, ob bmvew den Email-Versand-Auftrag publiziert hat
      assertEquals("Test 2", "1 els_pittner@els.dambach.de Dies ist Meldung -> 1", s);
    }
  }

  @Test(timeout = 10000)
  public void systemTest3() throws Exception
  {

    parametriereTest3();

    versendeMeldung(3, 2);

    Thread.sleep(2000);

    _resultList = new Vector<ResultData>();
    anmeldenMeldungsParameter("meldung.ArchivsystemMeldung3");
    anmeldenMeldungsZustand("meldung.ArchivsystemMeldung3");
    anmeldenMeldungsGruppen();

    // Warten bis Betriebsmeldungsverwaltung die Antwort publiziert hat
    synchronized (_resultList)
    {
      while (_resultList.size() < 6)
      {
        _resultList.wait();
      }

      for (int i = 0; i < _resultList.size(); i++)
      {
        ResultData rd = _resultList.get(i);
        DataDescription dd = rd.getDataDescription();
        SystemObject so = rd.getObject();
        // System.out.println("Test3: " + rd);
        // berprfen, ob bmvew die Daten der Meldung...
        if (dd.getAttributeGroup().getPid().equals("atg.meldung"))
        {

          // ... sowohl unter dem Aspekt parameterSoll
          if (dd.getAspect().getPid().equals("asp.parameterSoll"))
          {

            Data d = rd.getData();
            String s0 = d.getItem("Wichtigkeit").asTextValue().getText();
            String s1 = d.getItem("Status").asTextValue().getText();
            Array array = d.getItem("InformationsMeldungsAnteil").asArray();
            if (array.getLength() != 1)
              fail("Test 3-1");

            String s2 = array.getItem(0).getItem("MeldungsTyp").asTextValue().getText();
            String s3 = array.getItem(0).getItem("MeldungsKlasse").asTextValue().getText();
            String s4 = array.getItem(0).getItem("MeldungsStatus").asTextValue().getText();
            String s5 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();

            String s = s0 + " " + s1 + " " + s2 + " " + s3 + " " + s4 + " " + s5;

            assertEquals("Test 3", "5 neu Fach Warnung Meldung Dies ist Meldung -> 2", s);

          }// ... als auch unter dem Aspekt zustand publiziert
          else if (dd.getAspect().getPid().equals("asp.zustand"))
          {

            Data d = rd.getData();
            String s0 = d.getItem("Wichtigkeit").asTextValue().getText();
            String s1 = d.getItem("Status").asTextValue().getText();
            Array array = d.getItem("InformationsMeldungsAnteil").asArray();
            if (array.getLength() != 1)
              fail("Test 3-2");

            String s2 = array.getItem(0).getItem("MeldungsTyp").asTextValue().getText();
            String s3 = array.getItem(0).getItem("MeldungsKlasse").asTextValue().getText();
            String s4 = array.getItem(0).getItem("MeldungsStatus").asTextValue().getText();
            String s5 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();

            String s = s0 + " " + s1 + " " + s2 + " " + s3 + " " + s4 + " " + s5;

            assertEquals("Test 4", "5 neu Fach Warnung Meldung Dies ist Meldung -> 2", s);

          }
          else
            fail("Test 3-3");

        }// berprfen, ob bmvew die Zuordnung zur Meldungruppe publiziert
        else if (dd.getAttributeGroup().getPid().equals("atg.meldungsGruppeMeldung"))
        {

          if (so.getPid().equals("betriebsMeldung.meldungsGruppe1"))
          {

            Data d = rd.getData();
            Array array = d.getItem("MeldungReferenz").asArray();
            if (array.getLength() != 1){
              fail("Test 3-4");
            }

            String s = array.getItem(0).asReferenceValue().getSystemObjectPid();

            assertEquals("Test 5", "meldung.ArchivsystemMeldung3", s);

          }
          else
          {
            Data d = rd.getData();
            Array array = d.getItem("MeldungReferenz").asArray();
            assertTrue("Test6", array.getLength() == 0);
          }

        }
        else
          fail("Test 3-5");
      }
    }

    abmeldenMeldungsGruppen();
    abmeldenMeldungsZustand("meldung.ArchivsystemMeldung3");
  }

  @Test(timeout = 10000)
  public void systemTest4() throws Exception
  {

    parametriereTest4();

    anmeldenInformationsKanal("betriebsMeldung.informationsKanal");

    _resultList = new Vector<ResultData>();
    versendeMeldung(4, 3);

    // Warten bis Betriebsmeldungsverwaltung die Antwort publiziert hat
    synchronized (_resultList)
    {
      while (_resultList.size() < 1)
      {
        _resultList.wait();
      }

      ResultData rd = _resultList.get(0);

      // System.out.println(rd);

      Data d = rd.getData();
      String s0 = d.getItem("LaufendeNummer").asTextValue().getText();
      String s1 = d.getItem("ApplikationsKennung").asTextValue().getText();
      String s2 = d.getItem("ID").asTextValue().getText();
      String s3 = d.getItem("MeldungsTyp").asTextValue().getText();
      String s4 = d.getItem("MeldungsKlasse").asTextValue().getText();
      String s5 = d.getItem("Status").asTextValue().getText();
      String s6 = d.getItem("MeldungsText").asTextValue().getText();

      String s = s0 + " " + s1 + " " + s2 + " " + s3 + " " + s4 + " " + s5 + " " + s6;

      // berprfen, ob bmvew die Betriebsmeldung auf dem Informationskanal publiziert
      assertEquals("Test 7", "3 Archivsystem Meldung4 Fach Fatal Meldung Dies ist Meldung -> 3", s);
    }
  }

  @Test(timeout = 10000)
  public void systemTest5() throws Exception
  {

    parametriereTest5();

    anmeldenBetriebsMeldungSenke();

    _resultList = new Vector<ResultData>();
    versendeMeldung(5, 4);

    // Warten bis Betriebsmeldungsverwaltung die Antwort publiziert hat
    synchronized (_resultList)
    {
      while (_resultList.size() < 2)
      {
        _resultList.wait();
      }

      for (int i = 0; i < _resultList.size(); i++)
      {
        ResultData rd = _resultList.get(i);

        Data d = rd.getData();
        String s1 = d.getItem("ApplikationsKennung").asTextValue().getText();

        // berprfen, ob bmvew eine Betriebsmeldung versendet, wenn falscher Informationskanal parametriert wurde
        if (s1.equals("BetriebsMeldungApp"))
        {
          String s3 = d.getItem("MeldungsTyp").asTextValue().getText();
          String s4 = d.getItem("MeldungsKlasse").asTextValue().getText();
          String s5 = d.getItem("Status").asTextValue().getText();
          String s6 = d.getItem("MeldungsText").asTextValue().getText();
          String s = s1 + " " + s3 + " " + s4 + " " + s5 + " " + s6;
          // System.out.println(s);
          assertEquals(
              "Test 8",
              "BetriebsMeldungApp System Fehler Neue Meldung Informationskanal nicht vorhanden: betriebsMeldung.informationsKanalFalsch",
              s);

        }
      }

      abmeldenBetriebsMeldungSenke();
    }
  }

//  @Test(timeout = 10000)
  public void systemTest6() throws Exception
  {

    parametriereTest6();

    anmeldenBetriebsMeldungSenke2("kv.aoe.rp.nba.koeln.koblenz");

    _resultList = new Vector<ResultData>();
    versendeMeldung(6, 5);

    // Warten bis Betriebsmeldungsverwaltung die Antwort publiziert hat
    synchronized (_resultList)
    {
      while (_resultList.size() < 1)
      {
        _resultList.wait();
      }

      ResultData rd = _resultList.get(0);

      Data d = rd.getData();
      String s0 = d.getItem("LaufendeNummer").asTextValue().getText();
      String s1 = d.getItem("ApplikationsKennung").asTextValue().getText();
      String s2 = d.getItem("ID").asTextValue().getText();
      String s3 = d.getItem("MeldungsTyp").asTextValue().getText();
      String s4 = d.getItem("MeldungsKlasse").asTextValue().getText();
      String s5 = d.getItem("Status").asTextValue().getText();
      String s6 = d.getItem("MeldungsText").asTextValue().getText();

      String s = s0 + " " + s1 + " " + s2 + " " + s3 + " " + s4 + " " + s5 + " " + s6;

      // berprfen, ob bmvew die Betriebsmeldung an andere bmvew sendet
      assertEquals("Test 9", "5 Archivsystem Meldung6 Fach Fehler Meldung Dies ist Meldung -> 5", s);

      abmeldenBetriebsMeldungSenke2("kv.aoe.rp.nba.koeln.koblenz");

    }
  }

  @Test(timeout = 15000)
  public void systemTest7() throws Exception
  {

    versendeBenutzerAktionInBearbeitung();

    _resultList = new Vector<ResultData>();

    // Warten bis Betriebsmeldungsverwaltung die Antwort publiziert hat
    synchronized (_resultList)
    {
      while (_resultList.size() < 1)
      {
        _resultList.wait();
      }
    }

    versendeBenutzerAktionWiedervorlage(3);

    _resultList = new Vector<ResultData>();

    synchronized (_resultList)
    {
      while (_resultList.size() < 2)
      {
        _resultList.wait();
      }

      long l1 = 0L;
      for (int i = 0; i < _resultList.size(); i++)
      {
        ResultData rd = _resultList.get(i);
        Aspect asp = rd.getDataDescription().getAspect();

        // if (asp.getPid().equals("asp.parameterSoll"))
        {
          Data d = rd.getData();
          String s1 = d.getItem("Status").asTextValue().getText();

          if (s1.equals("Wiedervorlage"))
          {

            l1 = d.getItem("Wiedervorlage").asTimeValue().getMillis();

          }
          else if (s1.equals("in Bearbeitung"))
          {

            long l2 = rd.getDataTime();

            SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
            Date d1 = new Date();
            d1.setTime(l1);
            Date d2 = new Date();
            d2.setTime(l2);

            String x1 = sdf.format(d1);
            String x2 = sdf.format(d2);

            // System.out.println("10: " + x1 + " == " + x2);

            // berprfen, ob bmvew die Meldung nach eingestellter Wiedervorlagezeit erneut versendet
            assertEquals("Test 10", x1, x2);

          }
        }
      }
    }
  }

  @Test(timeout = 20000)
  public void systemTest8() throws Exception
  {

    versendeBenutzerAktionWiedervorlage(7);

    _resultList = new Vector<ResultData>();

    synchronized (_resultList)
    {
      while (_resultList.size() < 2)
      {
        _resultList.wait();
      }

      long l1 = 0L;

      for (int i = 0; i < _resultList.size(); i++)
      {
        ResultData rd = _resultList.get(i);
        Aspect asp = rd.getDataDescription().getAspect();

        // if (asp.getPid().equals("asp.parameterSoll"))
        {
          Data d = rd.getData();
          String s1 = d.getItem("Status").asTextValue().getText();

          if (s1.equals("Wiedervorlage"))
          {

            l1 = d.getItem("Wiedervorlage").asTimeValue().getMillis();

          }
          else if (s1.equals("in Bearbeitung"))
          {

            long l2 = rd.getDataTime();

            SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
            Date d1 = new Date();
            d1.setTime(l1);
            Date d2 = new Date();
            d2.setTime(l2);

            String x1 = sdf.format(d1);
            String x2 = sdf.format(d2);

            // System.out.println("11 : " + x1 + " == " + x2);

            // berprfen, ob bmvew die Meldung nach eingestellter Wiedervorlagezeit erneut versendet
            assertEquals("Test 11", x1, x2);

          }
        }
      }
    }
  }

  @Test(timeout = 10000)
  public void systemTest9() throws Exception
  {

    // abmeldenMeldungsZustand("meldung.ArchivsystemMeldung3");

    parametriereTest3();

    _resultList = new Vector<ResultData>();

    versendeMeldung(3, 10);
    versendeMeldung(3, 11);
    versendeMeldung(3, 12);
    versendeMeldung(3, 13);
    versendeMeldung(3, 14);
    versendeMeldung(3, 15);
    versendeMeldung(3, 16);
    versendeMeldung(3, 17);
    versendeMeldung(3, 18);
    versendeMeldung(3, 19);

    // Warten bis Betriebsmeldungsverwaltung die Antwort publiziert hat
    synchronized (_resultList)
    {
      while (_resultList.size() < 10)
      {
        _resultList.wait();
      }

      for (int i = 0; i < _resultList.size(); i++)
      {
        ResultData rd = _resultList.get(i);
        // Aspect asp = rd.getDataDescription().getAspect();
        

        // if (asp.getPid().equals("asp.parameterSoll"))
        {
          // berprfen, ob bmvew den Informationsanteil zum Meldungsobjekt inkrementiert.
          // Nach 10 Updates wird der lteste Anteil berschrieben

          Data d = rd.getData();
          Array array = d.getItem("InformationsMeldungsAnteil").asArray();
          
//          System.out.println(array.getLength());

          if (array.getLength() == 2)
          {            
            String s1 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();
            String s2 = array.getItem(1).getItem("MeldungsText").asTextValue().getText();
            assertEquals("Test 12", "Dies ist Meldung -> 10", s1);
            assertEquals("Test 13", "Dies ist Meldung -> 2", s2);
          }
          else if (array.getLength() == 3)
          {
            String s1 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();
            String s2 = array.getItem(1).getItem("MeldungsText").asTextValue().getText();
            String s3 = array.getItem(2).getItem("MeldungsText").asTextValue().getText();
            assertEquals("Test 14", "Dies ist Meldung -> 11", s1);
            assertEquals("Test 15", "Dies ist Meldung -> 10", s2);
            assertEquals("Test 16", "Dies ist Meldung -> 2", s3);
          }
          else if (array.getLength() == 4)
          {
            String s1 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();
            String s2 = array.getItem(1).getItem("MeldungsText").asTextValue().getText();
            String s3 = array.getItem(2).getItem("MeldungsText").asTextValue().getText();
            String s4 = array.getItem(3).getItem("MeldungsText").asTextValue().getText();
            assertEquals("Test 17", "Dies ist Meldung -> 12", s1);
            assertEquals("Test 18", "Dies ist Meldung -> 11", s2);
            assertEquals("Test 19", "Dies ist Meldung -> 10", s3);
            assertEquals("Test 20", "Dies ist Meldung -> 2", s4);
          }
          else if (array.getLength() == 5)
          {
            String s1 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();
            String s2 = array.getItem(1).getItem("MeldungsText").asTextValue().getText();
            String s3 = array.getItem(2).getItem("MeldungsText").asTextValue().getText();
            String s4 = array.getItem(3).getItem("MeldungsText").asTextValue().getText();
            String s5 = array.getItem(4).getItem("MeldungsText").asTextValue().getText();
            assertEquals("Test 21", "Dies ist Meldung -> 13", s1);
            assertEquals("Test 22", "Dies ist Meldung -> 12", s2);
            assertEquals("Test 23", "Dies ist Meldung -> 11", s3);
            assertEquals("Test 24", "Dies ist Meldung -> 10", s4);
            assertEquals("Test 25", "Dies ist Meldung -> 2", s5);
          }
          else if (array.getLength() == 6)
          {
            String s1 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();
            String s2 = array.getItem(1).getItem("MeldungsText").asTextValue().getText();
            String s3 = array.getItem(2).getItem("MeldungsText").asTextValue().getText();
            String s4 = array.getItem(3).getItem("MeldungsText").asTextValue().getText();
            String s5 = array.getItem(4).getItem("MeldungsText").asTextValue().getText();
            String s6 = array.getItem(5).getItem("MeldungsText").asTextValue().getText();
            assertEquals("Test 26", "Dies ist Meldung -> 14", s1);
            assertEquals("Test 27", "Dies ist Meldung -> 13", s2);
            assertEquals("Test 28", "Dies ist Meldung -> 12", s3);
            assertEquals("Test 29", "Dies ist Meldung -> 11", s4);
            assertEquals("Test 30", "Dies ist Meldung -> 10", s5);
            assertEquals("Test 31", "Dies ist Meldung -> 2", s6);

          }
          else if (array.getLength() == 7)
          {
            String s1 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();
            String s2 = array.getItem(1).getItem("MeldungsText").asTextValue().getText();
            String s3 = array.getItem(2).getItem("MeldungsText").asTextValue().getText();
            String s4 = array.getItem(3).getItem("MeldungsText").asTextValue().getText();
            String s5 = array.getItem(4).getItem("MeldungsText").asTextValue().getText();
            String s6 = array.getItem(5).getItem("MeldungsText").asTextValue().getText();
            String s7 = array.getItem(6).getItem("MeldungsText").asTextValue().getText();
            assertEquals("Test 32", "Dies ist Meldung -> 15", s1);
            assertEquals("Test 33", "Dies ist Meldung -> 14", s2);
            assertEquals("Test 34", "Dies ist Meldung -> 13", s3);
            assertEquals("Test 35", "Dies ist Meldung -> 12", s4);
            assertEquals("Test 36", "Dies ist Meldung -> 11", s5);
            assertEquals("Test 37", "Dies ist Meldung -> 10", s6);
            assertEquals("Test 38", "Dies ist Meldung -> 2", s7);
          }
          else if (array.getLength() == 8)
          {
            String s1 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();
            String s2 = array.getItem(1).getItem("MeldungsText").asTextValue().getText();
            String s3 = array.getItem(2).getItem("MeldungsText").asTextValue().getText();
            String s4 = array.getItem(3).getItem("MeldungsText").asTextValue().getText();
            String s5 = array.getItem(4).getItem("MeldungsText").asTextValue().getText();
            String s6 = array.getItem(5).getItem("MeldungsText").asTextValue().getText();
            String s7 = array.getItem(6).getItem("MeldungsText").asTextValue().getText();
            String s8 = array.getItem(7).getItem("MeldungsText").asTextValue().getText();
            assertEquals("Test 39", "Dies ist Meldung -> 16", s1);
            assertEquals("Test 40", "Dies ist Meldung -> 15", s2);
            assertEquals("Test 41", "Dies ist Meldung -> 14", s3);
            assertEquals("Test 42", "Dies ist Meldung -> 13", s4);
            assertEquals("Test 43", "Dies ist Meldung -> 12", s5);
            assertEquals("Test 44", "Dies ist Meldung -> 11", s6);
            assertEquals("Test 45", "Dies ist Meldung -> 10", s7);
            assertEquals("Test 46", "Dies ist Meldung -> 2", s8);
          }
          else if (array.getLength() == 9)
          {
            String s1 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();
            String s2 = array.getItem(1).getItem("MeldungsText").asTextValue().getText();
            String s3 = array.getItem(2).getItem("MeldungsText").asTextValue().getText();
            String s4 = array.getItem(3).getItem("MeldungsText").asTextValue().getText();
            String s5 = array.getItem(4).getItem("MeldungsText").asTextValue().getText();
            String s6 = array.getItem(5).getItem("MeldungsText").asTextValue().getText();
            String s7 = array.getItem(6).getItem("MeldungsText").asTextValue().getText();
            String s8 = array.getItem(7).getItem("MeldungsText").asTextValue().getText();
            String s9 = array.getItem(8).getItem("MeldungsText").asTextValue().getText();
            assertEquals("Test 47", "Dies ist Meldung -> 17", s1);
            assertEquals("Test 48", "Dies ist Meldung -> 16", s2);
            assertEquals("Test 49", "Dies ist Meldung -> 15", s3);
            assertEquals("Test 50", "Dies ist Meldung -> 14", s4);
            assertEquals("Test 51", "Dies ist Meldung -> 13", s5);
            assertEquals("Test 52", "Dies ist Meldung -> 12", s6);
            assertEquals("Test 53", "Dies ist Meldung -> 11", s7);
            assertEquals("Test 54", "Dies ist Meldung -> 10", s8);
            assertEquals("Test 55", "Dies ist Meldung -> 2", s9);
          }
          else if (array.getLength() == 10)
          {
            if (i == 8)
            {
              String s1 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();
              String s2 = array.getItem(1).getItem("MeldungsText").asTextValue().getText();
              String s3 = array.getItem(2).getItem("MeldungsText").asTextValue().getText();
              String s4 = array.getItem(3).getItem("MeldungsText").asTextValue().getText();
              String s5 = array.getItem(4).getItem("MeldungsText").asTextValue().getText();
              String s6 = array.getItem(5).getItem("MeldungsText").asTextValue().getText();
              String s7 = array.getItem(6).getItem("MeldungsText").asTextValue().getText();
              String s8 = array.getItem(7).getItem("MeldungsText").asTextValue().getText();
              String s9 = array.getItem(8).getItem("MeldungsText").asTextValue().getText();
              String s10 = array.getItem(9).getItem("MeldungsText").asTextValue().getText();
              assertEquals("Test 56", "Dies ist Meldung -> 18", s1);
              assertEquals("Test 57", "Dies ist Meldung -> 17", s2);
              assertEquals("Test 58", "Dies ist Meldung -> 16", s3);
              assertEquals("Test 59", "Dies ist Meldung -> 15", s4);
              assertEquals("Test 60", "Dies ist Meldung -> 14", s5);
              assertEquals("Test 61", "Dies ist Meldung -> 13", s6);
              assertEquals("Test 62", "Dies ist Meldung -> 12", s7);
              assertEquals("Test 63", "Dies ist Meldung -> 11", s8);
              assertEquals("Test 64", "Dies ist Meldung -> 10", s9);
              assertEquals("Test 65", "Dies ist Meldung -> 2", s10);
            }
            else if (i == 9)
            {
              String s1 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();
              String s2 = array.getItem(1).getItem("MeldungsText").asTextValue().getText();
              String s3 = array.getItem(2).getItem("MeldungsText").asTextValue().getText();
              String s4 = array.getItem(3).getItem("MeldungsText").asTextValue().getText();
              String s5 = array.getItem(4).getItem("MeldungsText").asTextValue().getText();
              String s6 = array.getItem(5).getItem("MeldungsText").asTextValue().getText();
              String s7 = array.getItem(6).getItem("MeldungsText").asTextValue().getText();
              String s8 = array.getItem(7).getItem("MeldungsText").asTextValue().getText();
              String s9 = array.getItem(8).getItem("MeldungsText").asTextValue().getText();
              String s10 = array.getItem(9).getItem("MeldungsText").asTextValue().getText();
              assertEquals("Test 66", "Dies ist Meldung -> 19", s1);
              assertEquals("Test 67", "Dies ist Meldung -> 18", s2);
              assertEquals("Test 68", "Dies ist Meldung -> 17", s3);
              assertEquals("Test 69", "Dies ist Meldung -> 16", s4);
              assertEquals("Test 70", "Dies ist Meldung -> 15", s5);
              assertEquals("Test 71", "Dies ist Meldung -> 14", s6);
              assertEquals("Test 72", "Dies ist Meldung -> 13", s7);
              assertEquals("Test 73", "Dies ist Meldung -> 12", s8);
              assertEquals("Test 74", "Dies ist Meldung -> 11", s9);
              assertEquals("Test 75", "Dies ist Meldung -> 10", s10);
            }
            else
              fail("Test 12-75");
          }

        }
      }
    }
  }

  @Test(timeout = 100000)
  public void systemTest10() throws Exception
  {

    // abmeldenMeldungsZustand("meldung.ArchivsystemMeldung3");

    parametriereTest3();

    _resultList = new Vector<ResultData>();

    versendeMeldung(10, 20);

    // Warten bis Betriebsmeldungsverwaltung die Antwort publiziert hat
    synchronized (_resultList)
    {
      while (_resultList.size() < 1)
      {
        _resultList.wait();
      }

      ResultData rd = _resultList.get(0);

      Data d = rd.getData();

      String s1 = d.getItem("Status").asTextValue().getText();
      Array array = d.getItem("InformationsMeldungsAnteil").asArray();

      String s2 = array.getItem(0).getItem("MeldungsTyp").asTextValue().getText();
      String s3 = array.getItem(0).getItem("MeldungsKlasse").asTextValue().getText();
      String s4 = array.getItem(0).getItem("MeldungsStatus").asTextValue().getText();
      String s5 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();

      String s = s1 + " " + s2 + " " + s3 + " " + s4 + " " + s5;

      // berprfen, ob bmvew eine quittierte Meldung auf den Zustand erledigt setzt
      // assertEquals("Test 76", "erledigt Fach Warnung Gutmeldung Dies ist Meldung -> 20", s);
 
      // berprfen, ob bmvew eine quittierte Meldung in dem Aktuellen Zustand (in Bearbeitung) belaesst.
      assertEquals("Test 76", "in Bearbeitung Fach Warnung Gutmeldung Dies ist Meldung -> 20", s);

    }
  }
  
  @Test(timeout = 15000)
  public void systemTest11() throws Exception
  {
    versendeBenutzerAktionErledigt();

    _resultList = new Vector<ResultData>();

    synchronized (_resultList)
    {
      while (_resultList.size() < 1)
      {
        _resultList.wait();
      }
      ResultData rd = _resultList.get(0);

      Data d = rd.getData();
      String s1 = d.getItem("Status").asTextValue().getText();
      
      Array array = d.getItem("BenutzerMeldungsAnteil").asArray();
//      for (int i = 0; i < array.getLength(); i++)
//      {
//        String s = array.getItem(i).getItem("MeldungsText").asTextValue().getText();
//        System.out.println(i + " ---------> MeldungsText: " + s);        
//      }

      String s2 = null;
      String s3 = null;
      String s4 = null;
      String s5 = null;

      if (array.getLength() == 4)
      {
        s2 = array.getItem(0).getItem("MeldungsText").asTextValue().getText();
        s3 = array.getItem(1).getItem("MeldungsText").asTextValue().getText();
        s4 = array.getItem(2).getItem("MeldungsText").asTextValue().getText();
        s5 = array.getItem(3).getItem("MeldungsText").asTextValue().getText();
      }else       
        fail("Test 77");
 
      String s = s1 + " " + s2 + " " + s3 + " " + s4+ " " + s5;
//
      // berprfen, ob bmvew eine quittierte Meldung auf den Zustand erledigt setzt
      // Auslesen der BenutzerMeldungsAnteile. Der letzte Eintrag steht im 1. Feldelement
      assertEquals("Test 77", "erledigt Aktion erledigt Aktion Wiedervorlage Aktion Wiedervorlage Aktion in Bearbeitung", s);     
    }

    anmeldenMeldungsGruppen();

    _removedList = new Vector<String>();

    synchronized (_removedList)
    {
      while (_removedList.size() < 1)
      {
        _removedList.wait();
      }      
    }

    _resultList = new Vector<ResultData>();

    synchronized (_resultList)
    {
      while (_resultList.size() < 1)
      {
        _resultList.wait();
      }

      ResultData rd = _resultList.get(0);
      Data d = rd.getData();
      Array array = d.getItem("MeldungReferenz").asArray();

      if (array.getLength() == 0)
      {

        SystemObject so = rd.getObject();

        // berprfen, ob bmvew die auf erledigt gesetzte Meldung nach dem Lschen aus der Meldungsgruppe entfernt
        assertEquals("Test 78", "betriebsMeldung.meldungsGruppe1", so.getPid());

      }
      else
        fail("Test 78");
    }
  }
  

  private void anmeldenFaxVersandAuftrag()
  {
    ArrayList<SystemObject> list = new ArrayList<SystemObject>();
    SystemObject so = _connection.getDataModel().getObject(_strVersandObjektKex);
    list.add(so);

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.faxVersandAuftrag");
    Aspect asp = _connection.getDataModel().getAspect("asp.auftrag");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, list, dd, ReceiveOptions.normal(), ReceiverRole.drain());
  }

  private void anmeldenEmailVersandAuftrag()
  {
    ArrayList<SystemObject> list = new ArrayList<SystemObject>();
    SystemObject so = _connection.getDataModel().getObject(_strVersandObjektKex);
    list.add(so);

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.emailVersandAuftrag");
    Aspect asp = _connection.getDataModel().getAspect("asp.auftrag");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, list, dd, ReceiveOptions.normal(), ReceiverRole.drain());
  }

  private void anmeldenMeldungsParameter(String pid)
  {
    ArrayList<SystemObject> list = new ArrayList<SystemObject>();
    SystemObject so = _connection.getDataModel().getObject(pid);
    list.add(so);

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.meldung");
    Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, list, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
  }

  private void anmeldenMeldungsZustand(String pid)
  {
    ArrayList<SystemObject> list = new ArrayList<SystemObject>();
    SystemObject so = _connection.getDataModel().getObject(pid);
    list.add(so);

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.meldung");
    Aspect asp = _connection.getDataModel().getAspect("asp.zustand");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, list, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
  }

  private void abmeldenMeldungsZustand(String pid)
  {
    ArrayList<SystemObject> list = new ArrayList<SystemObject>();
    SystemObject so = _connection.getDataModel().getObject(pid);
    list.add(so);

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.meldung");
    Aspect asp = _connection.getDataModel().getAspect("asp.zustand");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, list, dd);
  }

  private void anmeldenMeldungsGruppen()
  {
    SystemObjectType type = _connection.getDataModel().getType("typ.meldungsGruppe");
    List<SystemObject> gruppen = type.getElements();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.meldungsGruppeMeldung");
    Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, gruppen, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
  }

  private void abmeldenMeldungsGruppen()
  {
    SystemObjectType type = _connection.getDataModel().getType("typ.meldungsGruppe");
    List<SystemObject> gruppen = type.getElements();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.meldungsGruppeMeldung");
    Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, gruppen, dd);
  }

  private void anmeldenInformationsKanal(String pid)
  {
    ArrayList<SystemObject> list = new ArrayList<SystemObject>();
    SystemObject so = _connection.getDataModel().getObject(pid);
    list.add(so);

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.betriebsMeldung");
    Aspect asp = _connection.getDataModel().getAspect("asp.information");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, list, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
  }

  private void anmeldenBetriebsMeldungSenke()
  {
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.betriebsMeldung");
    Aspect asp = _connection.getDataModel().getAspect("asp.information");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, aoe, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
  }

  private void abmeldenBetriebsMeldungSenke()
  {
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.betriebsMeldung");
    Aspect asp = _connection.getDataModel().getAspect("asp.information");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, aoe, dd);
  }

  private void anmeldenBetriebsMeldungSenke2(String pid)
  {

    SystemObject aoe2 = _connection.getDataModel().getObject(pid);

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.betriebsMeldung");
    Aspect asp = _connection.getDataModel().getAspect("asp.information");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, aoe2, dd, ReceiveOptions.normal(), ReceiverRole.drain());
  }

  private void abmeldenBetriebsMeldungSenke2(String pid)
  {
    SystemObject aoe2 = _connection.getDataModel().getObject(pid);

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.betriebsMeldung");
    Aspect asp = _connection.getDataModel().getAspect("asp.information");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, aoe2, dd);
  }

  private void versendeMeldung(int nr, int cnt)
  {

    switch (nr)
    {

      case 1:
        String id = "Meldung1";
        instance.setApplicationLabel("Archivsystem");
        instance.sendMessage(
        // ID
            id,
            // Typ (System oder Fach)
            MessageType.APPLICATION_DOMAIN,
            // MeldungsTypZusatz
            "",
            // Klasse (Fatal, Fehler, Warnung und Information)
            MessageGrade.FATAL,
            // Konfigurationsobjekt auf das sich die Meldung bezieht
            aoe,
            // Zustand der Meldung (nderung, Gutmeldung, Meldung, Neu,
            // Wiederholung)
            MessageState.MESSAGE,
            // Urlasser
            causer,
            // Text
            "Dies ist Meldung -> " + cnt);
        break;

      case 2:
        id = "Meldung2";
        instance.sendMessage(
        // ID
            id,
            // Typ (System oder Fach)
            MessageType.APPLICATION_DOMAIN,
            // MeldungsTypZusatz
            "",
            // Klasse (Fatal, Fehler, Warnung und Information)
            MessageGrade.ERROR,
            // Konfigurationsobjekt auf das sich die Meldung bezieht
            aoe,
            // Zustand der Meldung (nderung, Gutmeldung, Meldung, Neu,
            // Wiederholung)
            MessageState.MESSAGE,
            // Urlasser
            causer,
            // Text
            "Dies ist Meldung -> " + cnt);
        break;

      case 3:
        id = "Meldung3";
        instance.sendMessage(
        // ID
            id,
            // Typ (System oder Fach)
            MessageType.APPLICATION_DOMAIN,
            // MeldungsTypZusatz
            "",
            // Klasse (Fatal, Fehler, Warnung und Information)
            MessageGrade.WARNING,
            // Konfigurationsobjekt auf das sich die Meldung bezieht
            aoe,
            // Zustand der Meldung (nderung, Gutmeldung, Meldung, Neu,
            // Wiederholung)
            MessageState.MESSAGE,
            // Urlasser
            causer,
            // Text
            "Dies ist Meldung -> " + cnt);
        break;

      case 4:
        id = "Meldung4";
        instance.sendMessage(
        // ID
            id,
            // Typ (System oder Fach)
            MessageType.APPLICATION_DOMAIN,
            // MeldungsTypZusatz
            "",
            // Klasse (Fatal, Fehler, Warnung und Information)
            MessageGrade.FATAL,
            // Konfigurationsobjekt auf das sich die Meldung bezieht
            aoe,
            // Zustand der Meldung (nderung, Gutmeldung, Meldung, Neu,
            // Wiederholung)
            MessageState.MESSAGE,
            // Urlasser
            causer,
            // Text
            "Dies ist Meldung -> " + cnt);
        break;

      case 5:
        id = "Meldung5";
        instance.sendMessage(
        // ID
            id,
            // Typ (System oder Fach)
            MessageType.APPLICATION_DOMAIN,
            // MeldungsTypZusatz
            "",
            // Klasse (Fatal, Fehler, Warnung und Information)
            MessageGrade.FATAL,
            // Konfigurationsobjekt auf das sich die Meldung bezieht
            aoe,
            // Zustand der Meldung (nderung, Gutmeldung, Meldung, Neu,
            // Wiederholung)
            MessageState.MESSAGE,
            // Urlasser
            causer,
            // Text
            "Dies ist Meldung -> " + cnt);
        break;

      case 6:
        id = "Meldung6";
        instance.setApplicationLabel("Archivsystem");
        instance.sendMessage(
        // ID
            id,
            // Typ (System oder Fach)
            MessageType.APPLICATION_DOMAIN,
            // MeldungsTypZusatz
            "",
            // Klasse (Fatal, Fehler, Warnung und Information)
            MessageGrade.ERROR,
            // Konfigurationsobjekt auf das sich die Meldung bezieht
            aoe,
            // Zustand der Meldung (nderung, Gutmeldung, Meldung, Neu,
            // Wiederholung)
            MessageState.MESSAGE,
            // Urlasser
            causer,
            // Text
            "Dies ist Meldung -> " + cnt);
        break;

      case 10:
        id = "Meldung3";
        instance.sendMessage(
        // ID
            id,
            // Typ (System oder Fach)
            MessageType.APPLICATION_DOMAIN,
            // MeldungsTypZusatz
            "",
            // Klasse (Fatal, Fehler, Warnung und Information)
            MessageGrade.WARNING,
            // Konfigurationsobjekt auf das sich die Meldung bezieht
            aoe,
            // Zustand der Meldung (nderung, Gutmeldung, Meldung, Neu,
            // Wiederholung)
            MessageState.GOOD_MESSAGE,
            // Urlasser
            causer,
            // Text
            "Dies ist Meldung -> " + cnt);
        break;

      default:
        System.out.println("Meldungsnummer nicht definiert :" + nr);

    }
  }

  private void parametriereTest1()
  {

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.betriebsMeldungsVerwaltungRegel");

    Data data = _connection.createData(atg);
    Array array = data.getItem("Regel").asArray();
    array.setLength(1);

    Data d0 = array.getItem(0);
    Array array2 = d0.getItem("Filter").asArray();
    array2.setLength(2);

    array2.getItem(0).getItem("Kriterium").asTextValue().setText("Meldungsklasse");
    array2.getItem(0).getItem("Operation").asTextValue().setText("Text gleich");
    Array array3 = array2.getItem(0).getItem("Wert").asArray();
    array3.setLength(1);
    array3.getItem(0).asTextValue().setText("Fatal");

    array2.getItem(1).getItem("Kriterium").asTextValue().setText("Applikation");
    array2.getItem(1).getItem("Operation").asTextValue().setText("Text gleich");
    Array array4 = array2.getItem(1).getItem("Wert").asArray();
    array4.setLength(1);
    array4.getItem(0).asTextValue().setText("Archivsystem");

    Array array5 = d0.getItem("Aktion").asArray();
    array5.setLength(1);
    array5.getItem(0).getItem("Aktion").asTextValue().setText("Fax");
    Array array6 = array5.getItem(0).getItem("Parameter").asArray();
    array6.setLength(1);
    array6.getItem(0).asTextValue().setText("+497222402209");

    parametriereObjekt("kv.aoe.nw.nba.koeln.koblenz", atg, data);

    try
    {
      Thread.sleep(3000);
    }
    catch (InterruptedException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }

  private void parametriereTest2()
  {

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.betriebsMeldungsVerwaltungRegel");

    Data data = _connection.createData(atg);
    Array array = data.getItem("Regel").asArray();
    array.setLength(1);

    Data d0 = array.getItem(0);
    Array array2 = d0.getItem("Filter").asArray();
    array2.setLength(1);

    array2.getItem(0).getItem("Kriterium").asTextValue().setText("Meldungsklasse");
    array2.getItem(0).getItem("Operation").asTextValue().setText("Text gleich");
    Array array3 = array2.getItem(0).getItem("Wert").asArray();
    array3.setLength(1);
    array3.getItem(0).asTextValue().setText("Fehler");

    Array array4 = d0.getItem("Aktion").asArray();
    array4.setLength(1);

    array4.getItem(0).getItem("Aktion").asTextValue().setText("Email");
    Array array5 = array4.getItem(0).getItem("Parameter").asArray();
    array5.setLength(1);
    array5.getItem(0).asTextValue().setText("els_pittner@els.dambach.de");

    parametriereObjekt("kv.aoe.nw.nba.koeln.koblenz", atg, data);

    try
    {
      Thread.sleep(3000);
    }
    catch (InterruptedException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }

  private void parametriereTest3()
  {

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.betriebsMeldungsVerwaltungRegel");

    Data data = _connection.createData(atg);
    Array array = data.getItem("Regel").asArray();
    array.setLength(1);

    Data d0 = array.getItem(0);
    Array array2 = d0.getItem("Filter").asArray();
    array2.setLength(1);

    array2.getItem(0).getItem("Kriterium").asTextValue().setText("Meldungsklasse");
    array2.getItem(0).getItem("Operation").asTextValue().setText("Text gleich");
    Array array3 = array2.getItem(0).getItem("Wert").asArray();
    array3.setLength(1);
    array3.getItem(0).asTextValue().setText("Warnung");

    Array array4 = d0.getItem("Aktion").asArray();
    array4.setLength(1);

    array4.getItem(0).getItem("Aktion").asTextValue().setText("Verwalten");
    Array array5 = array4.getItem(0).getItem("Parameter").asArray();
    array5.setLength(2);
    array5.getItem(0).asTextValue().setText("5");
    array5.getItem(1).asTextValue().setText("betriebsMeldung.meldungsGruppe1");

    parametriereObjekt("kv.aoe.nw.nba.koeln.koblenz", atg, data);

    try
    {
      Thread.sleep(3000);
    }
    catch (InterruptedException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }

  private void parametriereTest4()
  {

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.betriebsMeldungsVerwaltungRegel");

    Data data = _connection.createData(atg);
    Array array = data.getItem("Regel").asArray();
    array.setLength(1);

    Data d0 = array.getItem(0);
    Array array2 = d0.getItem("Filter").asArray();
    array2.setLength(1);

    array2.getItem(0).getItem("Kriterium").asTextValue().setText("Meldungsklasse");
    array2.getItem(0).getItem("Operation").asTextValue().setText("Text gleich");
    Array array3 = array2.getItem(0).getItem("Wert").asArray();
    array3.setLength(1);
    array3.getItem(0).asTextValue().setText("Fatal");

    Array array4 = d0.getItem("Aktion").asArray();
    array4.setLength(1);

    array4.getItem(0).getItem("Aktion").asTextValue().setText("Verteilen");
    Array array5 = array4.getItem(0).getItem("Parameter").asArray();
    array5.setLength(1);
    array5.getItem(0).asTextValue().setText("betriebsMeldung.informationsKanal");

    parametriereObjekt("kv.aoe.nw.nba.koeln.koblenz", atg, data);

    try
    {
      Thread.sleep(3000);
    }
    catch (InterruptedException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }

  private void parametriereTest5()
  {

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.betriebsMeldungsVerwaltungRegel");

    Data data = _connection.createData(atg);
    Array array = data.getItem("Regel").asArray();
    array.setLength(1);

    Data d0 = array.getItem(0);
    Array array2 = d0.getItem("Filter").asArray();
    array2.setLength(1);

    array2.getItem(0).getItem("Kriterium").asTextValue().setText("Meldungsklasse");
    array2.getItem(0).getItem("Operation").asTextValue().setText("Text gleich");
    Array array3 = array2.getItem(0).getItem("Wert").asArray();
    array3.setLength(1);
    array3.getItem(0).asTextValue().setText("Fatal");

    Array array4 = d0.getItem("Aktion").asArray();
    array4.setLength(1);

    array4.getItem(0).getItem("Aktion").asTextValue().setText("Verteilen");
    Array array5 = array4.getItem(0).getItem("Parameter").asArray();
    array5.setLength(1);
    array5.getItem(0).asTextValue().setText("betriebsMeldung.informationsKanalFalsch");

    parametriereObjekt("kv.aoe.nw.nba.koeln.koblenz", atg, data);

    try
    {
      Thread.sleep(3000);
    }
    catch (InterruptedException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }

  private void parametriereTest6()
  {

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.betriebsMeldungsVerwaltungRegel");

    Data data = _connection.createData(atg);
    Array array = data.getItem("Regel").asArray();
    array.setLength(1);

    Data d0 = array.getItem(0);
    Array array2 = d0.getItem("Filter").asArray();
    array2.setLength(1);

    array2.getItem(0).getItem("Kriterium").asTextValue().setText("Meldungsklasse");
    array2.getItem(0).getItem("Operation").asTextValue().setText("Text gleich");
    Array array3 = array2.getItem(0).getItem("Wert").asArray();
    array3.setLength(1);
    array3.getItem(0).asTextValue().setText("Fehler");

    Array array4 = d0.getItem("Aktion").asArray();
    array4.setLength(1);

    array4.getItem(0).getItem("Aktion").asTextValue().setText("Senden");
    Array array5 = array4.getItem(0).getItem("Parameter").asArray();
    array5.setLength(1);
    array5.getItem(0).asTextValue().setText("kv.aoe.rp.nba.koeln.koblenz");

    parametriereObjekt("kv.aoe.nw.nba.koeln.koblenz", atg, data);

    try
    {
      Thread.sleep(3000);
    }
    catch (InterruptedException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }

  private void versendeBenutzerAktionInBearbeitung()
  {

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.meldungsBearbeitung");
    Aspect asp = _connection.getDataModel().getAspect("asp.vorgabe");

    Data data = _connection.createData(atg);
    Array array = data.getItem("MeldungReferenz").asArray();
    array.setLength(1);

    SystemObject so = _connection.getDataModel().getObject("meldung.ArchivsystemMeldung3");
    array.getItem(0).asReferenceValue().setSystemObject(so);

    data.getItem("Wichtigkeit").asTextValue().setText("6");
    data.getItem("Status").asTextValue().setText("in Bearbeitung");

    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    Date d = null;
    try
    {
      d = sdf.parse("01.01.2009 00:00:00,000");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    data.getItem("Wiedervorlage").asTimeValue().setMillis(d.getTime());
    data.getItem("BenutzerMeldungsAnteil").asTextValue().setText("Aktion in Bearbeitung");

    // data.getItem("Wiedervorlage").asTimeValue().setMillis(d.getTime() + 10000);

    Sender sender = new Sender(_connection);
    sender.sende(aoe, atg, asp, data, SenderRole.sender());

  }
  
  private void versendeBenutzerAktionErledigt()
  {
    
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.meldungsBearbeitung");
    Aspect asp = _connection.getDataModel().getAspect("asp.vorgabe");
    
    Data data = _connection.createData(atg);
    Array array = data.getItem("MeldungReferenz").asArray();
    array.setLength(1);
    
    SystemObject so = _connection.getDataModel().getObject("meldung.ArchivsystemMeldung3");
    array.getItem(0).asReferenceValue().setSystemObject(so);
    
    data.getItem("Wichtigkeit").asTextValue().setText("10");
    data.getItem("Status").asTextValue().setText("erledigt");
    
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    Date d = null;
    try
    {
      d = sdf.parse("01.01.2009 00:00:00,000");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    data.getItem("Wiedervorlage").asTimeValue().setMillis(d.getTime());
    data.getItem("BenutzerMeldungsAnteil").asTextValue().setText("Aktion erledigt");
    
    // data.getItem("Wiedervorlage").asTimeValue().setMillis(d.getTime() + 10000);
    
    Sender sender = new Sender(_connection);
    sender.sende(aoe, atg, asp, data, SenderRole.sender());
    
  }

  private void versendeBenutzerAktionWiedervorlage(int sekundenBisWiedervorlage)
  {

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.meldungsBearbeitung");
    Aspect asp = _connection.getDataModel().getAspect("asp.vorgabe");

    Data data = _connection.createData(atg);
    Array array = data.getItem("MeldungReferenz").asArray();
    array.setLength(1);

    SystemObject so = _connection.getDataModel().getObject("meldung.ArchivsystemMeldung3");
    array.getItem(0).asReferenceValue().setSystemObject(so);

    data.getItem("Wichtigkeit").asTextValue().setText("6");
    data.getItem("Status").asTextValue().setText("Wiedervorlage");

    Date d = new Date();
    data.getItem("Wiedervorlage").asTimeValue().setMillis(d.getTime() + sekundenBisWiedervorlage * 1000);
    data.getItem("BenutzerMeldungsAnteil").asTextValue().setText("Aktion Wiedervorlage");

    Sender sender = new Sender(_connection);
    sender.sende(aoe, atg, asp, data, SenderRole.sender());

  }

  private void parametriereObjekt(String pid, AttributeGroup atg, Data parameter)
  {

    // Erzeuge Parameter
    Aspect asp = _connection.getDataModel().getAspect("asp.parameterVorgabe");
    VerwaltungDynObj vewParam = new VerwaltungDynObj(_connection, _connection.getDataModel(), null, null, null, atg,
        asp);

    Data[] datas = new Data[1];
    datas[0] = parameter;
    vewParam.setDynamicObject(_connection.getDataModel().getObject(pid));
    vewParam.parametriere(datas);
  }

  @Override
  public void update(ResultData[] results)
  {
    // TODO Auto-generated method stub
    for (ResultData result : results)
    {
      if (result.getData() != null)
      {

        DataDescription dd = result.getDataDescription();
        AttributeGroup atg = dd.getAttributeGroup();

        if (atg.getPid().equals("atg.faxVersandAuftrag") || atg.getPid().equals("atg.emailVersandAuftrag"))
        {
          synchronized (_resultList)
          {
            _resultList.add(result);
            _resultList.notify();
          }
        }
        else if (atg.getPid().equals("atg.meldung") || atg.getPid().equals("atg.meldungsGruppeMeldung"))
        {
          synchronized (_resultList)
          {
            _resultList.add(result);
            _resultList.notify();
          }
        }
        else if (atg.getPid().equals("atg.betriebsMeldung"))
        {
          synchronized (_resultList)
          {
            _resultList.add(result);
            _resultList.notify();
          }
        }
        else
          System.out.println(result);

      }
      else
      {
        // System.out.println("Null: " + result.getObject() + " " +
        // result.getDataDescription());
      }
    }
  }

  @Override
  public void update(MutableSet set, SystemObject[] addedObjects, SystemObject[] removedObjects)
  {
    // TODO Auto-generated method stub
    if (set.getName().equals("Meldungen"))
    {

      if (addedObjects != null)
      {
        for (int i = 0; i < addedObjects.length; i++)
        {
          SystemObject so = addedObjects[i];
          // anmeldenMeldungsParameter(so.getPid());
          // System.out.println("added - " + addedObjects[i]);
        }
      }

      if (removedObjects != null)
      {
        for (int i = 0; i < removedObjects.length; i++)
        {
          // System.out.println("removed - " + removedObjects[i]);

          synchronized (_removedList)
          {
            _removedList.add(removedObjects[i].getPid());
            _removedList.notify();
          }

        }

      }

    }

  }

  private static void loescheDynamischesObjekt(String pid)
  {
    ConfigurationArea ca = _connection.getDataModel().getConfigurationArea(_strKonfigBereich);
    ConfigurationObject co = (ConfigurationObject)aoe;
    DynamicObjectType dot = null;
    AttributeGroup atgKonfig = null;
    dot = (DynamicObjectType)_connection.getDataModel().getType("typ.meldung");
    atgKonfig = _connection.getDataModel().getAttributeGroup("atg.meldungEigenschaften");

    Aspect aspKonfig = _connection.getDataModel().getAspect("asp.eigenschaften");
    VerwaltungDynObj vewKonfig = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgKonfig,
        aspKonfig);

    vewKonfig.setDynamicObject(_connection.getDataModel().getObject(pid));
    vewKonfig.loesche(pid, "Meldungen");
  }

}
